import json

import requests

from .actions import Actions


class APIException(Exception):
    pass


class HTTPException(Exception):
    pass


class AsanakAPI(object):
    def __init__(
        self,
        api_key: str = "",
        username: str = "",
        password: str = "",
        version="v2rest",
    ):
        self.version = version
        self.host = f"sms.asanak.ir/webservice/{self.version}"
        self.username, self.password = (
            (username, password) if username and password else api_key.split(",")
        )
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "charset": "utf-8",
        }

    def _normalize_params(self, params={}):
        if isinstance(params, dict):
            return json.dumps(
                {"username": self.username, "password": self.password, **params}
            )
        elif isinstance(params, list):
            return json.dumps(
                {
                    "username": self.username,
                    "password": self.password,
                    "data": [param for param in params],
                }
            )
        else:
            return json.dumps(
                {
                    "username": self.username,
                    "password": self.password,
                    **params.__dict__,
                }
            )

    def _request(self, action, parameters={}):
        url = "https://" + self.host + "/" + action["name"]
        try:
            res = requests.post(url, headers=self.headers, data=parameters)
            content = res.content
            try:
                response = json.loads(content.decode())
                if res.status_code == 200:
                    response = {
                        "status": response["meta"][action["success_state_field"]],
                        "message": response["meta"]["message"],
                        "data": response["data"],
                    }
                else:
                    response = {
                        "status": res.status_code,
                        "message": response["meta"]["message"],
                        "data": response["data"],
                    }
                return response
            except ValueError as e:
                raise HTTPException(e)
            return response
        except requests.exceptions.RequestException as e:
            raise HTTPException(e)

    def sms_send(self, params):
        return self._request(Actions.SENDSMS, self._normalize_params(params))

    def sms_template(self, params):
        return self._request(Actions.TEMPLATE, self._normalize_params(params))

    def sms_p2p(self, params):
        return self._request(Actions.P2P, self._normalize_params(params))

    def sms_status(self, params):
        res = self._request(Actions.STATUS, self._normalize_params(params))
        return res

    def sms_template_list(self):
        return self._request(Actions.TEMPLATELIST, self._normalize_params())

    def get_credit(self):
        return self._request(Actions.GETCREDEIT, self._normalize_params())

    def get_rial_credit(self):
        return self._request(Actions.GETRIALCREDIT, self._normalize_params())

    def __repr__(self):
        return "asanak.AsanakAPI({!r})".format(self.username)

    def __str__(self):
        return "asanak.AsanakAPI({!s})".format(self.username)



SMS_STATUS = {
    "NotFound": -1,
    "InQueue": 1,
    "Pending": 4,
    "Sent": 2,
    "NoResponse": 7,
    "Reject": 8,
    "PartiallySent": 10,
    "SentToDest": 9,
    "Success": 6,
    "Failed": 5,
    "NoMsgID": 11,
    "PartiallySuccess": 12,
    "NoDelivery": 13,
}


API_STATUS = {
    "200.1010": "InvalidDestination",
    "200.1002": "InvalidSourceNumber",
    "200.200": "Success",
    "200.0": "Success",
    "400.1001": "InvalidDestination",
    "400.1002": "InvalidSourceNumber",
    "400.1012": "InvalidMessage",
    "400.1003": "InvalidMsgid",
    "400": "BadRequest",
    "403.1001": "InvalidTemplate",
    "406.1002": "InvalidSourceNumber",
    "402.1006": "InsufficientCredit",
    "401.1009": "InvalidCredentials",
    "401": "InvalidCredentials",
    "401.1012": "InvalidCredentials",
    "500.1004": "ServerError",
    "500.500": "ServerError",
    "500": "ServerError",
    "404.404": "NotFound",
}



class Actions:
    SENDSMS = {
        "name": "sendsms",
        "success_state_field": "status",
        "execption_state_field": "status",
    }
    TEMPLATE = {
        "name": "template",
        "success_state_field": "status",
        "execption_state_field": "status",
    }
    P2P = {
        "name": "p2psendsms",
        "success_state_field": "status",
        "execption_state_field": "status",
    }

    STATUS = {
        "name": "msgstatus",
        "success_state_field": "code",
        "execption_state_field": "status",
    }
    TEMPLATELIST = {
        "name": "templatelist",
        "success_state_field": "status",
        "execption_state_field": "code",
    }

    GETCREDEIT = {
        "name": "getcredit",
        "success_state_field": "status",
        "execption_state_field": "status",
    }

    GETRIALCREDIT = {
        "name": "getrialcredit",
        "success_state_field": "status",
        "execption_state_field": "status",
    }
